@@ -1,5 +1,5 @@
 /*
- * $Id: control.c,v 1.112 2003/10/09 01:25:54 matto Exp $
+ * $Id: control.c,v 1.112.4.1 2003/10/31 03:54:30 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -788,33 +788,48 @@ void DoSelectTypeAction
                 break;
 
 #ifdef HAVE_ROTATE
-            case SELECT_ROTATE_LEFT:
-                switch ( Prefs()->rotate ) {
+            case SELECT_ROTATE_LEFT: {
+                RotateType angle;
+                angle = Prefs()->rotate;
+                switch ( angle ) {
                     case ROTATE_ZERO:
-                        Prefs()->rotate = ROTATE_PLUS90;
+                        angle = ROTATE_PLUS90;
                         break;
                     case ROTATE_PLUS90:
-                        Prefs()->rotate = ROTATE_MINUS90;
+                        angle = ROTATE_MINUS90;
                         break;
                     case ROTATE_MINUS90:
-                        Prefs()->rotate = ROTATE_ZERO;
+                        angle = ROTATE_ZERO;
                 }
+                if ( Prefs()->individualDocumentFonts )
+                    GetHistoryPtr()->rotate = angle;
+                else
+                    Prefs()->rotate = angle;
                 FrmUpdateForm( Prefs()->lastForm, frmRedrawUpdateCode );
                 break;
+            }
 
-            case SELECT_ROTATE_RIGHT:
-                switch ( Prefs()->rotate ) {
+            case SELECT_ROTATE_RIGHT: {
+                RotateType angle;
+                angle = Prefs()->rotate;
+                switch ( angle ) {
                     case ROTATE_PLUS90:
-                        Prefs()->rotate = ROTATE_ZERO;
+                        angle = ROTATE_ZERO;
                         break;
                     case ROTATE_MINUS90:
-                        Prefs()->rotate = ROTATE_PLUS90;
+                        angle = ROTATE_PLUS90;
                         break;
                     case ROTATE_ZERO:
-                        Prefs()->rotate = ROTATE_MINUS90;
+                        angle = ROTATE_MINUS90;
+                        break;
                 }
+                if ( Prefs()->individualDocumentFonts )
+                    GetHistoryPtr()->rotate = angle;
+                else
+                    Prefs()->rotate = angle;
                 FrmUpdateForm( Prefs()->lastForm, frmRedrawUpdateCode );
                 break;
+            }
 #endif
 
             case SELECT_TOGGLE_TOOLBAR:
