@@ -1,5 +1,5 @@
 /*
- * $Id: document.c,v 1.131.4.3 2003/10/14 17:28:12 nordstrom Exp $
+ * $Id: document.c,v 1.131.4.8 2003/11/28 15:26:29 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -598,6 +598,9 @@ static void AdjustVerticalOffset
     /* Adjust the record */
     newRecord = *meta;
     /* bottom of page */
+
+    /** FIX ME: This code assumes that no non-final fragment can be less
+        than one screenful long. **/
     if ( newRecord.verticalOffset + adjustment < minVerticalOffset ) {
         if ( adjustment < 0 ) {
             /* at the bottom of the page (and beyond) */
@@ -626,7 +629,8 @@ static void AdjustVerticalOffset
         else
             continuation = true;
     }
-    else if ( 0 < newRecord.verticalOffset + adjustment ) {
+    if ( nextRecordId == NO_RECORD &&
+         0 < newRecord.verticalOffset + adjustment ) {
         /* top of page or beyond */
         if ( 0 < adjustment ) {
             nextRecordId = GetSequentialRecordId( DIRECTION_UP );
@@ -638,8 +642,7 @@ static void AdjustVerticalOffset
                 /* if we're already at the top of the page
                    we can't scroll any further */
                 if ( 0 <= newRecord.verticalOffset ) {
-                    MemHandleUnlock( metaRecord );
-                    return;
+                    adjustment = 0;
                 }
                 else {
                     /* scroll to the top */
@@ -650,6 +653,12 @@ static void AdjustVerticalOffset
         else
             continuation = true;
     }
+
+    if ( adjustment == 0 ) {
+        MemHandleUnlock( metaRecord );
+        return;
+    }
+
     adjustAmount                = abs( adjustment );
     newRecord.verticalOffset   += adjustment;
     SetHistoryVerticalOffset( newRecord.verticalOffset );
@@ -697,7 +706,7 @@ static void AdjustVerticalOffset
 #endif
 
     if ( nextRecordId != NO_RECORD ) {
-        ViewRecordAt( nextRecordId, true, 0, 0, yPosition, backwards, true,
+        ViewRecordAt( nextRecordId, false, 0, 0, yPosition, backwards, true,
             DrawChar );
     }
 }
@@ -779,7 +788,7 @@ static YOffset CalculatePosition
             thisContext.foundYPosition        = -1;
             thisContext.writeMode             = NoDraw;
             DrawParagraph( &thisContext, paragraph, record );
-            if ( 0 < thisContext.foundYPosition ) {
+            if ( 0 <= thisContext.foundYPosition ) {
                 offsetInParagraph = yPosition - thisContext.foundYPosition;
                 pOffset           = i;
             }
@@ -1054,6 +1063,7 @@ void DoPageMove
     if ( RotExtentY() < abspixels ) {
         recordId  = DeltaToRecordId( pixels, &cOffset );
         if ( recordId != currentRecordId ) {
+            PalmSetCoordinateSystem( prevCoordSys );
             ViewRecord( recordId, false, NO_OFFSET, cOffset );
             return;
         }
