@@ -1,5 +1,5 @@
 /*
- * $Id: fullscreenform.c,v 1.27 2003/09/28 23:01:40 chrish Exp $
+ * $Id: fullscreenform.c,v 1.27.4.5 2003/11/14 16:52:23 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -86,8 +86,10 @@ static Int16         tableX;
 static Int16         tableY;
 static Boolean       penDownInFrame;
 static Boolean       fullscreenformActive = false;
-static RectangleType fullBounds;     
+static RectangleType fullBounds;
 static Boolean       fastEnough;	/* SuperVZ or better */
+static Coord         originalBackButtonHeight = 0;
+static Coord         originalBackButtonWidth = 0;
 
 
 /***********************************************************************
@@ -97,7 +99,8 @@ static Boolean       fastEnough;	/* SuperVZ or better */
  ***********************************************************************/
 static Boolean FsFormInit( void ) FULLSCREENFORM_SECTION;
 static void MoveImage( Coord x, Coord y ) FULLSCREENFORM_SECTION;
-static void DrawImage( ImageType* image, Coord drawX, Coord drawY, RectangleType* bounds ) FULLSCREENFORM_SECTION;
+static void DrawImage( ImageType* image, Coord drawX, Coord drawY,
+    RectangleType* bounds ) FULLSCREENFORM_SECTION;
 static void DrawScanner( void ) FULLSCREENFORM_SECTION;
 static void DrawButton( void ) FULLSCREENFORM_SECTION;
 static void FreeMemory( void ) FULLSCREENFORM_SECTION;
@@ -116,7 +119,9 @@ static Boolean FsFormInit( void )
     MemHandle       imageHandle;
     Err             err;
     Coord           maxFrameSize;
-    UInt32 processorType;
+    UInt32          processorType;
+    UInt16          backButtonIndex;
+    RectangleType   newBounds;
 
     err = FtrGet( sysFileCSystem, sysFtrNumProcessorID, &processorType );
     if ( err == errNone && processorType >= 0x00040000 )
@@ -136,12 +141,24 @@ static Boolean FsFormInit( void )
 
     fsForm = FrmGetFormPtr( GetValidForm( frmFullscreen ) );
 
-    FrmDrawForm( fsForm );
-
     screenWindow = WinGetDrawWindow();
-    WinGetDrawWindowBounds( &screenBounds );
+
+    screenBounds.topLeft.x = 0;
+    screenBounds.topLeft.y = 0;
+    screenBounds.extent.x  = RotMaxExtentX();
+    screenBounds.extent.y  = RotMaxExtentY();
+    newBounds              = screenBounds;
+    if ( fastEnough )
+        RotateRectangleInPlace( &newBounds );
+    WinSetBounds( screenWindow, &newBounds );
     HiResAdjustBounds( &screenBounds, palmHiRes );
 
+    backButtonIndex = FrmGetObjectIndex( fsForm, frmFullscreenBack );
+
+    FrmHideObject( fsForm, backButtonIndex );
+
+    FrmDrawForm( fsForm );
+
     bufferWindow = NULL;
     if ( Support35() ) { /* 3.0 (at least) mis-behaves terribly */
         bufferWindow = WinCreateOffscreenWindow( screenBounds.extent.x,
@@ -172,32 +189,48 @@ static Boolean FsFormInit( void )
         frameBounds.extent.y = (UInt32) imageBounds.extent.y *
             (UInt32) frameBounds.extent.x / (UInt32) imageBounds.extent.x;
     }
-    frameBounds.topLeft.x = MaxExtentX() - frameBounds.extent.x - 1;
-    frameBounds.topLeft.y = MaxExtentY() - frameBounds.extent.y - 1;
+    frameBounds.topLeft.x = RotMaxExtentX() - frameBounds.extent.x - 1;
+    frameBounds.topLeft.y = RotMaxExtentY() - frameBounds.extent.y - 1;
 
-    FrmGetObjectBounds( fsForm, FrmGetObjectIndex( fsForm, frmFullscreenBack ),
-                        &ctlBounds );
+    /* adjust position of back button for displays taller than 160 */
+    if ( 0 == originalBackButtonHeight ) {
+        FrmGetObjectBounds( fsForm, backButtonIndex, &ctlBounds );
+        HiResAdjustBounds( &ctlBounds, sonyHiRes );
+        originalBackButtonWidth  = ctlBounds.extent.x;
+        originalBackButtonHeight = ctlBounds.extent.y;
+    }
+    ctlBounds.topLeft.x = 0;
+    ctlBounds.topLeft.y = RotMaxExtentY() - originalBackButtonHeight;
+    ctlBounds.extent.x  = originalBackButtonWidth;
+    ctlBounds.extent.y  = originalBackButtonHeight;
 
     if ( fastEnough ) {
-        CtlGlueSetFont( GetObjectPtr( frmFullscreenBack ), symbolFont );
+        ControlType*    backButton;
+        backButton = GetObjectPtr( frmFullscreenBack );
+        CtlGlueSetFont( backButton, symbolFont );
         switch ( Prefs()->rotate ) {
             case ROTATE_PLUS90:
-                CtlSetLabel( GetObjectPtr( frmFullscreenBack ), "\006" );
+                CtlSetLabel( backButton, "\006" );
                 break;
             case ROTATE_MINUS90:
-                CtlSetLabel( GetObjectPtr( frmFullscreenBack ), "\005" );
+                CtlSetLabel( backButton, "\005" );
                 break;
             case ROTATE_ZERO:
             default:
-                CtlSetLabel( GetObjectPtr( frmFullscreenBack ), "\003" );
+                CtlSetLabel( backButton, "\003" );
                 break;
         }
-        FrmGetObjectBounds( fsForm, FrmGetObjectIndex( fsForm,
-                            frmFullscreenBack ), &ctlBounds );
         RotateRectangleInPlace( &ctlBounds );
-        FrmSetObjectBounds( fsForm, FrmGetObjectIndex( fsForm,
-                            frmFullscreenBack ), &ctlBounds );
     }
+    if ( IsHiResTypeSony( HiResType() ) ) {
+        ctlBounds.topLeft.x /= 2;
+        ctlBounds.topLeft.y /= 2;
+        ctlBounds.extent.x  /= 2;
+        ctlBounds.extent.y  /= 2;
+    }
+    FrmSetObjectBounds( fsForm, backButtonIndex, &ctlBounds );
+
+    FrmShowObject( fsForm, backButtonIndex );
 
     ctlBounds.topLeft.x--;
     ctlBounds.topLeft.y--;
@@ -270,16 +303,17 @@ static void MoveImage
         screenWindow = WinSetDrawWindow( bufferWindow );
 
     DrawImage( image, -x, -y, &fullBounds );
-    if ( Prefs()->visualAid ) {
-        DrawButton();
-        DrawScanner();
-    }
 
     if ( bufferWindow != NULL ) {
         WinSetDrawWindow( screenWindow );
         WinCopyRectangle( bufferWindow, screenWindow, &bufferBounds, 0, 0,
             winPaint );
     }
+
+    if ( Prefs()->visualAid ) {
+        DrawButton();
+        DrawScanner();
+    }
 }
 
 
@@ -332,24 +366,38 @@ static void DrawImage
             Coord           offsetY;
 
             RctGetIntersection( bounds, &screenBounds, &useMe);
-            if ( useMe.extent.x && useMe.extent.y ) {
-                offsetX = useMe.topLeft.x - screenBounds.topLeft.x;
-                offsetY = useMe.topLeft.y - screenBounds.topLeft.y;
-                drawWindow = WinCreateOffscreenWindow( useMe.extent.x,
-                             useMe.extent.y, screenFormat, &error );
-                screenWindow = WinSetDrawWindow( drawWindow );
-                WinDrawBitmap( thisImage->bitmap, drawX - offsetX,
-                               drawY - offsetY );
-                bitmap = WinGetBitmap( drawWindow );
-                WinSetDrawWindow( screenWindow );
-                if ( fastEnough )
-                    RotDrawBitmap( bitmap, offsetX, offsetY );
-                else
-                    GeneralWinDrawBitmap( bitmap, offsetX, offsetY );
-                WinDeleteWindow( drawWindow, false );
+            if ( 0 < useMe.extent.x && 0 < useMe.extent.y ) {
+                if ( ! Support35() ) {
+                    WinDrawBitmap( thisImage->bitmap, drawX, drawY );
+                }
+                else {
+                    /* crop the bitmap so that the rotation code only
+                       rotates the visible part */
+                    offsetX = useMe.topLeft.x - screenBounds.topLeft.x;
+                    offsetY = useMe.topLeft.y - screenBounds.topLeft.y;
+                    drawWindow = WinCreateOffscreenWindow( useMe.extent.x,
+                                     useMe.extent.y, screenFormat, &error );
+                    if ( error == errNone ) {
+                        screenWindow = WinSetDrawWindow( drawWindow );
+                        WinDrawBitmap( thisImage->bitmap, drawX - offsetX,
+                                       drawY - offsetY );
+                        bitmap = WinGetBitmap( drawWindow );
+                        WinSetDrawWindow( screenWindow );
+                        if ( fastEnough )
+                            RotDrawBitmap( bitmap, offsetX, offsetY );
+                        else
+                            GeneralWinDrawBitmap( bitmap, offsetX, offsetY );
+                        WinDeleteWindow( drawWindow, false );
+                    }
+                    else {
+                        if ( fastEnough )
+                            RotDrawBitmap( thisImage->bitmap, drawX, drawY );
+                        else
+                            GeneralWinDrawBitmap( thisImage->bitmap, drawX, drawY );
+                    }
+                }
             }
         }
-
         /* Specifically erase any whitespace beyound the image's bounds */
         if ( imageBounds.extent.x < screenBounds.extent.x ) {
             whitespace.topLeft.y = 0;
@@ -373,7 +421,6 @@ static void DrawImage
             else
                 WinEraseRectangle( &whitespace, 0 );
         }
-
         PalmSetCoordinateSystem( prevCoordSys );
     }
 }
@@ -853,10 +900,10 @@ Boolean FsFormHandleEvent
 
     penX = event->screenX;
     penY = event->screenY;
-    if ( fastEnough )
-        RotFromScreenXY( &penX, &penY );
     HiResAdjust( &penX, sonyHiRes );
     HiResAdjust( &penY, sonyHiRes );
+    if ( fastEnough )
+        RotFromScreenXY( &penX, &penY );
 
     switch ( event->eType ) {
         case ctlSelectEvent:
@@ -899,6 +946,15 @@ Boolean FsFormHandleEvent
             break;
 #endif
 
+        case frmUpdateEvent:
+            FreeMemory();
+            menuActive = false;
+            if ( FsFormInit() == false ) {
+                /* Do something */
+            }
+            handled = true;
+            break;
+
         case frmOpenEvent:
             menuActive = false;
             if ( FsFormInit() == false ) {
