@@ -1,5 +1,5 @@
 /*
- * $Id: paragraph.c,v 1.121 2003/10/08 15:54:29 matto Exp $
+ * $Id: paragraph.c,v 1.121.4.6 2003/11/26 03:10:20 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -353,10 +353,8 @@ static Boolean IsSoftHyphen
     WChar token
     )
 {
-    if ( Prefs()->enableSoftHyphens )
-        return ( token == SOFT_HYPHEN );
-    else
-        return false;
+    return ( token == SOFT_HYPHEN ) && Prefs()->enableSoftHyphens &&
+               DeviceUses8BitChars();
 }
 
 
@@ -1546,7 +1544,7 @@ static Boolean GetLineMetrics
 
         prevPosition  = pContext->position;
 
-        if ( DeviceUses8BitChars() && 
+        if ( DeviceUses8BitChars() &&
              *prevPosition != 0 && pushedChar == 0 &&
              prevPosition < pContext->last &&
              ! GrayFntIsCurrentGray()
@@ -1820,7 +1818,9 @@ static Boolean GetLineMetrics
                 savedFixedWidthFont    = fixedWidthFont;
                 savedPrevFontHeight    = GetPrevFontHeight();
             }
-            savedPContext.position = prevPosition;
+            else {
+                savedPContext.position = pContext->position;
+            }
             savedSubHeight         = subHeight;
         }
     }
@@ -1917,7 +1917,7 @@ static void WriteLine
 
     AlignText( tContext, pContext );
 
-    while ( pContext->position < until ) {
+    while ( pContext->position < until || pushedChar != 0 ) {
         TokenType nextTokenType;
         WChar     nextChar;
         Int16     imageWidth;
@@ -2118,7 +2118,6 @@ void DrawParagraph
         cachedParagraph = paragraphNum;
         inCache = true;
     }
-
     for ( ;; ) {
         prevPosition = pContext.position - pContext.recordTop;
         if ( handleCaching )
