@@ -1,5 +1,5 @@
 /*
- * $Id: rotate.c,v 1.22 2003/09/28 23:01:40 chrish Exp $
+ * $Id: rotate.c,v 1.22.4.1 2003/11/13 15:17:09 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -77,6 +77,7 @@ typedef struct {
  ***********************************************************************/
 static void GlueDrawPixel( Coord x, Coord y ) ROTATE_SECTION;
 static void GlueErasePixel( Coord x, Coord y ) ROTATE_SECTION;
+static UInt8 GetMaxBitDepthForBitmap( UInt32 numPixels ) ROTATE_SECTION;
 static void RotDrawCharsGeneral( Char* string, Int16 length,
     Coord x, Coord y, Boolean invert ) ROTATE_SECTION;
 static void RotateXY( Coord x, Coord y ) ROTATE_SECTION;
@@ -270,6 +271,31 @@ static void GlueErasePixel
 
 
 
+/* Get the highest screen depth possible in the
+   context of a <= 64000 byte bitmap.  If the background is white, this
+   is 1.  Return 0 if no depth works. */
+static UInt8 GetMaxBitDepthForBitmap
+    (
+    UInt32 numPixels
+    )
+{
+    UInt8 bitmapDepth;
+    
+    bitmapDepth = Prefs()->screenDepth;
+
+    while ( 64000l * 8 < numPixels * bitmapDepth ) {
+        bitmapDepth >>= 1;
+    }
+
+    return bitmapDepth;
+}
+
+
+
+
+
+
+
 /* Draw a rotated character or string at (x,y). */
 /* On OS3.5+ this calls RotBitmapMinus90 and hence only supports minus
    90 degree rotation. */
@@ -341,14 +367,26 @@ static void RotDrawCharsGeneral
         Boolean          convertedSrc;
         BitmapType*      oldSrcBitmap;
         WinHandle        oldWindow;
+        UInt8            bitmapDepth;
 
         clockwise = ( Prefs()->rotate == ROTATE_MINUS90 );
 
         width = FntCharsWidth( string, length );
-        srcBitmap = BmpCreate( width, height - charTop, 1, NULL, &err );
+        
+        if ( HaveWhiteBackground() ) {
+            bitmapDepth = 1;
+        }
+        else {
+            bitmapDepth = GetMaxBitDepthForBitmap( ( UInt32 ) width * 
+                              ( height - charTop ) );
+            if ( 0 == bitmapDepth )
+                return;
+        }
+
+        srcBitmap = BmpCreate( width, height - charTop, bitmapDepth, NULL, &err );
         if ( err != errNone )
             return;
-        destBitmap = BmpCreate( height - charTop, width, 1, NULL, &err );
+        destBitmap = BmpCreate( height - charTop, width, bitmapDepth, NULL, &err );
         if ( err != errNone ) {
             BmpDelete( srcBitmap );
             return;
@@ -397,7 +435,7 @@ static void RotDrawCharsGeneral
             armData.src          = BmpGetBits( srcBitmap );
             armData.srcRowBytes  = srcRowBytes;
             armData.clockwise    = clockwise;
-            armData.bitDepth     = 1;
+            armData.bitDepth     = bitmapDepth;
 
             PceNativeCall( RotateBitmapArmlet, &armData );
         }
@@ -406,7 +444,7 @@ static void RotDrawCharsGeneral
             RotateBitmap( width, height - charTop,
                 BmpGetBits( destBitmap ), destRowBytes,
                 BmpGetBits( srcBitmap ), srcRowBytes,
-                clockwise, 1 );
+                clockwise, bitmapDepth );
 
         bounds.topLeft.x = x;
         bounds.topLeft.y = y + charTop;
@@ -443,7 +481,7 @@ static void RotDrawCharsGeneral
     glyphPlusCharTop = glyph + rowWords * charTop;
 
     while ( 0 < length ) {
-         Boolean missing;
+        Boolean missing;
 
         charWidth  = TxtGlueGetNextChar( string, 0, &ch );
         string    += charWidth;
